#!/usr/bin/perl
#
# This script creates <VirtualHost> containers for default sites on the fly.
# It creates *-placeholder sites and one for each bound IPv4 and IPv6 IP.
# For the HTTPS VirtualHosts it uses the AdmServ SSL certificate.
#

$DEBUG = "0";

use Apache2::ServerUtil();
use lib qw(/usr/sausalito/perl);

use CCE;
use Sauce::Service;

$cce = new CCE;
$cce->connectuds();

($oid) = $cce->find('System');
($ok, $System) = $cce->get($oid);

# Get Server FQDN:
$System_FQDN = $System->{'hostname'} . '.' . $System->{'domainname'};

# Check if $System_FQDN starts with "localhost"
if ($System_FQDN =~ /^localhost/) {
    # Replace $System_FQDN with the web server IP from the environment
    $System_FQDN = $ENV{'SERVER_ADDR'};
}

# Get "System" . "Web":
($ok, $objWeb) = $cce->get($oid, 'Web');

($ok, $objNginx) = $cce->get($oid, 'Nginx');

# Get "System" . "SSL":
($ok, $objSSL) = $cce->get($oid, 'SSL');

# Check if we're using our custom /etc/httpd/conf.d/php.conf:
$custom_php_conf = `cat /etc/httpd/conf.d/php.conf |grep "Please note: BlueOnyx 5210R sets all PHP related"|wc -l`;
chomp($custom_php_conf);
if ($custom_php_conf ne "1") {
  if (-f "$confdir/php.conf.bx") {
    system("cp $confdir/php.conf.bx $confdir/php.conf");
  }
  else {
    open(my $fh, '>', '/etc/httpd/conf.d/php.conf');
    print $fh "# Please note: BlueOnyx 5210R sets all PHP related options\n";
    print $fh "# in the Vsite include files. Please do not edit this file\n";
    print $fh "# to add custom options, as they will be deleted.\n";
    close $fh;
  }
  system("chown root:root /etc/httpd/conf.d/php.conf");
  Sauce::Service::service_run_init('httpd', 'reload');
}

# Find out if we do have a CA-Cert:
@ca_certs = $cce->scalar_to_array($objSSL->{caCerts});
$CALine = '';
if ( -f "/etc/admserv/certs/ca-certs") {
    $check = `cat /etc/admserv/certs/ca-certs |grep 'BEGIN CERTIFICATE-'|wc -l`;
    chomp($check);
    if ($check eq "1") {
        # We do have intermediate(s). Use it:
        $CALine = 'SSLCACertificateFile  /etc/admserv/certs/ca-certs';
    }    
}

# Server-Name:
$ServerName = $System->{hostname} . '.' . $System->{domainname};

# HTTP and SSL ports:
$httpPort = "80";
if ($objWeb->{'httpPort'}) {
    $httpPort = $objWeb->{'httpPort'};
}
$sslPort = "443";
if ($objWeb->{'sslPort'}) {
    $sslPort = $objWeb->{'sslPort'};
}

# Set up default error pages:
if (!-d '/var/www/html/error') {
    system("mkdir /var/www/html/error");
    system("cp /etc/skel/vsite/en/web/error/*.html /var/www/html/error/")
}

if (-f '/etc/skel/vsite/en/web/error/502-bad-gateway.html') {
    system("cp /etc/skel/vsite/en/web/error/502-bad-gateway.html /var/www/html/error/")
}

$CipherLine = "        SSLProtocol -all +TLSv1.3 +TLSv1.2\n";
$CipherLine .= "        SSLCipherSuite    TLSv1.3   TLS_CHACHA20_POLY1305_SHA256:TLS_AES_256_GCM_SHA384:TLS_AES_128_GCM_SHA256\n";
$CipherLine .= "        SSLCipherSuite    SSL       ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:EECDH+AESGCM:EDH+AESGCM\n";
$CipherLine .= "        SSLOpenSSLConfCmd Curves X25519:secp521r1:secp384r1:prime256v1\n";
$CipherLine .= "        SSLHonorCipherOrder On\n";

# GUI may run on different ports and the admin URLs may be nonstandard. We pull them from CODB and use them:
$GUI_PORT = $System->{'GUI_PORT'};
@GUI_URLs = $cce->scalar_to_array($System->{'GUI_URLs'});
$gui_url_rewrites = "    RewriteEngine On\n";
$gui_url_rewrites .= '    RewriteCond %{HTTP_HOST}                ^([^:]+)' . "\n";
$gui_url_rewrites .= '    RewriteCond %{DOCUMENT_ROOT}            !-d' . "\n";
$gui_url_rewrites .= '    RewriteRule .*                          https://%1:' . $GUI_PORT . '/gui/Forbidden403 [L,R]' . "\n";
foreach my $uri (@GUI_URLs) {
    if ($System->{'GUIredirects'} eq '1') {
        $gui_url_rewrites .= '    RewriteCond %{HTTP_HOST}                ^([^:]+)' . "\n";
        $gui_url_rewrites .= '    RewriteRule ^/' . $uri . '/?$                  https://' . $System_FQDN . ':' . $GUI_PORT . '/login' . ' [L,R]' . "\n";
    }
    else {
        $gui_url_rewrites .= '    RewriteCond %{HTTP_HOST}                ^([^:]+)' . "\n";
        $gui_url_rewrites .= '    RewriteRule ^/' . $uri . '/?$                  https://%1:' . $GUI_PORT . '/login' . ' [L,R]' . "\n";
    }
}

# Generate config on the fly:
$config = '';
$config = <<CONFIG;
    #NameVirtualHost *:$httpPort
    #NameVirtualHost *:$sslPort

    Listen 443 https

$gui_url_rewrites

$CipherLine

    SSLStrictSNIVHostCheck off
    SSLCompression off
    
    SSLUseStapling          on
    SSLStaplingResponderTimeout 5
    SSLStaplingReturnResponderErrors off
    SSLStaplingCache        shmcb:/var/run/ocsp(128000)

    SSLOptions +FakeBasicAuth +ExportCertData +StrictRequire

    <VirtualHost *:$httpPort>
        ServerName $ServerName
        ServerAdmin admin
        DocumentRoot /var/www/html/

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>

    <VirtualHost *:$sslPort>
        ServerName $ServerName
        SSLengine on

        $CipherLine

        SSLCompression off

        # Server Certificate:
        $CALine
        SSLCertificateFile    /etc/admserv/certs/certificate
        SSLCertificateKeyFile /etc/admserv/certs/key
        ServerAdmin admin
        DocumentRoot /var/www/html

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>
CONFIG

# Note: Presence of the file '/etc/sysconfig/guissl' will disable default Vsite for all IPs:
if (! -f '/etc/sysconfig/guissl') {
    # Get all IPs:
    @arr_assigned_ipv4 = split (/\n/, `LC_ALL=C /sbin/ip address show |grep inet|grep global|awk -F "inet " '{print \$2}'|awk -F " brd " '{print \$1}'|cut -d / -f1|sed '/^\$/d'`);
    @arr_assigned_ipv6 = split (/\n/, `LC_ALL=C /sbin/ip address show |grep inet|grep global|awk -F "inet6 " '{print \$2}'|awk -F " brd " '{print \$1}'|cut -d / -f1|sed '/^\$/d'`);
}
else {
    @arr_assigned_ipv4 = ();
    @arr_assigned_ipv6 = ();
}

foreach $x (@arr_assigned_ipv4) {
    $config .= <<CONFIG;
    <VirtualHost $x:$httpPort>
        ServerName $ServerName
        ServerAdmin admin
        DocumentRoot /var/www/html

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>

    <VirtualHost $x:$sslPort>
        ServerName $ServerName
        SSLengine on
        SSLCompression off

        $CipherLine

        # Server Certificate:
        $CALine
        SSLCertificateFile    /etc/admserv/certs/certificate
        SSLCertificateKeyFile /etc/admserv/certs/key
        ServerAdmin admin
        DocumentRoot /var/www/html

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>
CONFIG
}

foreach $z (@arr_assigned_ipv6) {
    $config .= <<CONFIG;
    <VirtualHost [$z]:$httpPort>
        ServerName $ServerName
        ServerAdmin admin
        DocumentRoot /var/www/html/

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>

    <VirtualHost [$z]:$sslPort>
        ServerName $ServerName
        SSLengine on
        SSLCompression off

        $CipherLine

        # Server Certificate:
        $CALine
        SSLCertificateFile    /etc/admserv/certs/certificate
        SSLCertificateKeyFile /etc/admserv/certs/key
        ServerAdmin admin
        DocumentRoot /var/www/html

        ErrorDocument 401 /error/401-authorization.html
        ErrorDocument 403 /error/403-forbidden.html
        ErrorDocument 404 /error/404-file-not-found.html
        ErrorDocument 500 /error/500-internal-server-error.html
    </VirtualHost>
CONFIG
}

# For debugging:
if ($DEBUG) {
    open($fh, '>', '/tmp/httpd.debug');
    print $fh $config;
    close $fh;
}

#
### Disable if Nginx SSL-Proxy is enabled:
#

if ($objNginx->{enabled} eq '1') {
    # We disable by zapping the config - EXCEPT for the $gui_url_rewrites:
    $config = $gui_url_rewrites;
}

# Push out config:
$server = Apache2::ServerUtil->server;
$server->add_config([split /\n/, $config]);

$cce->bye('SUCCESS');

1;

# 
# Copyright (c) 2018-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2018-2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
